@@ -100,13 +100,15 @@ wnck_window_is_decorated (WnckWindow *window)
 {
   GdkDisplay *display = gdk_display_get_default();
   Atom hints_atom = None;
-  guchar *data;
+  guchar *data = NULL;
   MotifWmHints *hints = NULL;
-  Atom type;
+  Atom type = None;
   gint format;
   gulong nitems;
   gulong bytes_after;
   gboolean retval;
+
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
   
   hints_atom = gdk_x11_get_xatom_by_name_for_display (display, 
                                                       _XA_MOTIF_WM_HINTS);
@@ -117,36 +119,33 @@ wnck_window_is_decorated (WnckWindow *window)
 		                  False, AnyPropertyType, &type, &format, &nitems,
 		                  &bytes_after, &data);
   
+  if (type == None || !data) return TRUE;
+  
   hints = (MotifWmHints *)data; 
   
-  if (type == None)
-    retval = TRUE;
-  else
-  {
-	
-//    if (hints->flags & MWM_HINTS_DECORATIONS)
-  //  {
-    //  hints->flags |= MWM_HINTS_DECORATIONS;
-      retval = hints->decorations;
-   // }
-  }
+  retval = hints->decorations;
   
-  XFree (hints);
+  if (data)
+    XFree (data);
+
   return retval;
 }
 
 static void
 gdk_window_set_mwm_hints (WnckWindow *window,
-			                    MotifWmHints *new_hints)
+                          MotifWmHints *new_hints)
 {
   GdkDisplay *display = gdk_display_get_default();
   Atom hints_atom = None;
-  guchar *data;
-  MotifWmHints *hints;
-  Atom type;
+  guchar *data = NULL;
+  MotifWmHints *hints = NULL;
+  Atom type = None;
   gint format;
   gulong nitems;
   gulong bytes_after;
+
+  g_return_if_fail (WNCK_IS_WINDOW (window));
+  g_return_if_fail (GDK_IS_DISPLAY (display));
   
   hints_atom = gdk_x11_get_xatom_by_name_for_display (display, 
                                                       _XA_MOTIF_WM_HINTS);
@@ -157,7 +156,7 @@ gdk_window_set_mwm_hints (WnckWindow *window,
 		                  False, AnyPropertyType, &type, &format, &nitems,
 		                  &bytes_after, &data);
   
-  if (type == None)
+  if (type != hints_atom || !data)
     hints = new_hints;
   else
   {
@@ -175,31 +174,34 @@ gdk_window_set_mwm_hints (WnckWindow *window,
     }
   }
   
-  gdk_error_trap_push ();
-  XChangeProperty (GDK_DISPLAY_XDISPLAY (display), wnck_window_get_xid (window),
+  _wnck_error_trap_push ();
+  XChangeProperty (GDK_DISPLAY_XDISPLAY (display), 
+                   wnck_window_get_xid (window),
                    hints_atom, hints_atom, 32, PropModeReplace,
                    (guchar *)hints, sizeof (MotifWmHints)/sizeof (long));
   gdk_flush ();
-  gdk_error_trap_pop ();
+  _wnck_error_trap_pop ();
   
-  if (hints != new_hints)
-    XFree (hints);
+  if (data)
+    XFree (data);
 }
 
 static void
 _window_set_decorations (WnckWindow      *window,
 			                   GdkWMDecoration decorations)
 {
-  MotifWmHints hints;
+  MotifWmHints *hints;
   
   g_return_if_fail (WNCK_IS_WINDOW (window));
   
   /* initialize to zero to avoid writing uninitialized data to socket */
-  memset(&hints, 0, sizeof(hints));
-  hints.flags = MWM_HINTS_DECORATIONS;
-  hints.decorations = decorations;
+  hints = g_slice_new0 (MotifWmHints);
+  hints->flags = MWM_HINTS_DECORATIONS;
+  hints->decorations = decorations;
  
-  gdk_window_set_mwm_hints (window, &hints);
+  gdk_window_set_mwm_hints (window, hints);
+
+  g_slice_free (MotifWmHints, hints);
 }
 
 /* </TAKEN FROM GDK> */
@@ -210,15 +212,42 @@ window_is_too_large_for_screen (WnckWindow *window)
   static GdkScreen *screen = NULL;
   gint x=0, y=0, w=0, h=0;
 
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
+
   if (screen == NULL)
     screen = gdk_screen_get_default ();
-
+    
   wnck_window_get_geometry (window, &x, &y, &w, &h);
-  if (w > (gdk_screen_get_width (screen) + 20) //some wiggle room
-      || h > (gdk_screen_get_height (screen)+20))
-  {
-    return TRUE;
-  }
+  
+  /* some wiggle room */
+  return (screen && 
+          (w > (gdk_screen_get_width (screen) + 20) ||
+           h > (gdk_screen_get_height (screen)+20)));
+}
+
+static gboolean
+on_window_maximised_changed (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
+
+  if (window_is_too_large_for_screen (window))
+    {
+      _window_set_decorations (window, 1);
+      wnck_window_unmaximize (window);
+    }
+  else
+    {
+      _window_set_decorations (window, 0);
+    }
+  return FALSE;
+}
+
+static gboolean
+enable_window_decorations (WnckWindow *window)
+{
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
+
+  _window_set_decorations (window, 1);
   return FALSE;
 }
 
@@ -232,26 +261,17 @@ on_window_state_changed (WnckWindow      *window,
 
   if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "exclude"))==1)
     return;
-
+  
   if (change_mask & WNCK_WINDOW_STATE_MAXIMIZED_HORIZONTALLY
       || change_mask & WNCK_WINDOW_STATE_MAXIMIZED_VERTICALLY)
   {
     if (wnck_window_is_maximized (window) && app->priv->undecorate)
     {
-      if (window_is_too_large_for_screen (window))
-      {
-        /* The window is too big for the screen, so redecorate */
-        _window_set_decorations (window, 1);
-        wnck_window_unmaximize (window);
-      }
-      else
-      {    
-        _window_set_decorations (window, 0);
-      }
+      g_idle_add ((GSourceFunc)on_window_maximised_changed, window);
     }
     else
     {
-      _window_set_decorations (window, 1);
+      g_idle_add ((GSourceFunc)enable_window_decorations, window);
     }
   }
 }
@@ -260,7 +280,6 @@ static gboolean
 is_excluded (MaximusApp *app, WnckWindow *window)
 {
   MaximusAppPrivate *priv;
-  WnckClassGroup *group;
   WnckWindowType type;
   WnckWindowActions actions;
   gchar *res_name;
@@ -278,12 +297,6 @@ is_excluded (MaximusApp *app, WnckWindow *window)
 
   if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "exclude"))==1)
     return TRUE;
-  
-  group = wnck_window_get_class_group (window);
-  
-  _wnck_get_wmclass (wnck_window_get_xid (window), &res_name, &class_name);
-
-  g_debug ("Window opened: res_name=%s -- class_name=%s", res_name, class_name);
 
   /* Ignore if the window is already fullscreen */
   if (wnck_window_is_fullscreen (window))
@@ -291,7 +304,7 @@ is_excluded (MaximusApp *app, WnckWindow *window)
     g_debug ("Excluding (is fullscreen): %s\n",wnck_window_get_name (window));
     return TRUE;
   }
-
+  
   /* Make sure the window supports maximising */
   actions = wnck_window_get_actions (window);
   if (actions & WNCK_WINDOW_ACTION_RESIZE
@@ -301,6 +314,10 @@ is_excluded (MaximusApp *app, WnckWindow *window)
     ; /* Is good to maximise */
   else
     return TRUE;
+
+  _wnck_get_wmclass (wnck_window_get_xid (window), &res_name, &class_name);
+
+  g_debug ("Window opened: res_name=%s -- class_name=%s", res_name, class_name);
  
   /* Check internal list of class_ids */
   for (i = 0; i < G_N_ELEMENTS (default_exclude_classes); i++)
@@ -314,7 +331,7 @@ is_excluded (MaximusApp *app, WnckWindow *window)
       return TRUE;
     } 
   }
-  
+
   /* Check user list */
   for (c = priv->exclude_class_list; c; c = c->next)
   {
@@ -426,7 +443,8 @@ on_exclude_class_changed (GConfClient        *client,
   g_return_if_fail (MAXIMUS_IS_APP (app));
   priv = app->priv;
 
-  g_slist_free (priv->exclude_class_list);
+  if (priv->exclude_class_list)
+    g_slist_free (priv->exclude_class_list);
   
   priv->exclude_class_list= gconf_client_get_list (client, 
                                                    APP_EXCLUDE_CLASS, 
@@ -437,6 +455,8 @@ on_exclude_class_changed (GConfClient        *client,
 static gboolean
 show_desktop (WnckScreen *screen)
 {
+  g_return_val_if_fail (WNCK_IS_SCREEN (screen), FALSE);
+  
   wnck_screen_toggle_showing_desktop (screen, TRUE);
   return FALSE;
 }
@@ -452,6 +472,7 @@ on_app_undecorate_changed (GConfClient        *client,
     
   g_return_if_fail (MAXIMUS_IS_APP (app));
   priv = app->priv;
+  g_return_if_fail (WNCK_IS_SCREEN (priv->screen));
 
   priv->undecorate = gconf_client_get_bool (client, 
                                             APP_UNDECORATE, 
